package org.maltparser.core.dataformat;

import java.net.URL;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.maltparser.core.exception.MaltChainedException;
import org.maltparser.core.helper.Util;
import org.maltparser.core.symbol.SymbolTableHandler;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 *  
 *
 * @author Johan Hall
 * @since 1.0
**/
public class DataFormatSpecification {	
	protected Logger logger;
	protected int entryPositionCounter;
	protected String name;
	protected SortedSet<DataFormatEntry> entries;
	
	public DataFormatSpecification(Logger logger) {
		this.logger = logger;
		entries = new TreeSet<DataFormatEntry>();
		entryPositionCounter = 0;
	}
	
	public DataFormatInstance createDataFormatInstance(SymbolTableHandler symbolTables, String specialSymbolsfileName, String rootLabel) throws MaltChainedException {
		return new DataFormatInstance(entries, symbolTables, specialSymbolsfileName, rootLabel);
	}
		
	public void parseDataFormatXMLfile(String fileName) throws MaltChainedException {
		parseDataFormatXMLfile(Util.findURL(fileName));
	}
	
	private void parseDataFormatXMLfile(URL url) throws MaltChainedException {
		if (url == null) {
			throw new DataFormatException("The data format specifcation file cannot be found. ");
		}
		
		if (logger.isInfoEnabled()) {
			logger.debug("Loading data format specification...\n");
		}
		
        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db = dbf.newDocumentBuilder();

    		Element root = db.parse(url.openStream()).getDocumentElement();
    		if (root.getNodeName().equals("dataformat")) { 
    			name = root.getAttribute("name");
    		} else {
    			throw new DataFormatException("Data format specification file must contain one 'dataformat' element. ");
    		}
    		NodeList cols = root.getElementsByTagName("column");
            Element col = null;
            for (int i = 0; i < cols.getLength(); i++) {
            	col = (Element)cols.item(i);
            	entries.add(new DataFormatEntry(i, col.getAttribute("name"), col.getAttribute("category"),col.getAttribute("type"), col.getAttribute("default")));
             }
        } catch (java.io.IOException e) {
        	throw new DataFormatException("Cannot find the file "+url.toString()+". ", e);
        } catch (ParserConfigurationException e) {
        	throw new DataFormatException("Problem parsing the file "+url.toString()+". ", e);
        } catch (SAXException e) {
        	throw new DataFormatException("Problem parsing the file "+url.toString()+". ", e);
        }
	}
	
	public void addEntry(String name, String category, String type, String defaultOutput) {
		entries.add(new DataFormatEntry(entryPositionCounter++, name, category, type, defaultOutput));
	}
	
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("Data format specification: ");
		sb.append(name);
		sb.append('\n');
		for (DataFormatEntry dfe : entries) {
			sb.append(dfe);
			sb.append('\n');
		}
		return sb.toString();
	}
}
