package org.maltparser.core.sentence;


import org.apache.log4j.Logger;
import org.maltparser.core.dataformat.DataFormatInstance;
import org.maltparser.core.exception.MaltChainedException;

/**
Sentence consists of a sequence of tokens.

@author Johan Hall

@since 1.0
*/
public class Sentence {
	public static final int DEFAULTMAXSENLEN = 256;
	private DataFormatInstance dataFormatInstance;
	private Logger logger;
	private Token[] tokens;
	private int capacity;
	private int size;
	
	public Sentence(Logger logger, DataFormatInstance dataFormatInstance) throws MaltChainedException {
		this(logger, dataFormatInstance, DEFAULTMAXSENLEN);
	}
	
	public Sentence(Logger logger, DataFormatInstance dataFormatInstance, int capacity) throws MaltChainedException {
		setLogger(logger);
		setCapacity(capacity);
		setDataFormatInstance(dataFormatInstance);
		this.tokens = new Token[capacity];
		this.size = 0;
		
		for (int i=1;i<capacity;i++) {
			this.tokens[i] = new Token(i, logger, dataFormatInstance);
		}
	}
	
	public void readSentence(String[] tokenLines) throws MaltChainedException {
		int i=0;
		size = 1; 
		for (;i<tokenLines.length;i++,size++) {
			if (tokenLines[i] == null) {
				break;
			}
			if (size < capacity) {
				tokens[size].setColumnInstances(tokenLines[i]);
			}
		}
	}
	
	
	public int getCapacity() {
		return capacity;
	}

	public void setCapacity(int value) {		
		this.capacity = value;	
	}
	
	public int getSize() {
		return size;
	}
	
	
	public DataFormatInstance getDataFormatInstance() {
		return dataFormatInstance;
	}

	public void setDataFormatInstance(DataFormatInstance dataFormatInstance) {
		this.dataFormatInstance = dataFormatInstance;
	}

	public Logger getLogger() {
		return logger;
	}

	public void setLogger(Logger logger) {
		this.logger = logger;
	}

	public Token[] getTokens() {
		return tokens;
	}
		
	public Token getToken(int index) {
		if (index > 0 && index <= capacity) {
			return tokens[index];
		} 
		return null;
	}
	
	public void setToken(String line) throws MaltChainedException {
		size++;
		if (size < capacity) {
			tokens[size].setColumnInstances(line);
		} else {
			throw new SentenceException("The capacity ("+capacity+") has been exceeded. ");
		}
	}
	
	public boolean hasTokens() {
		if (size == 0) {
			return false;
		}
		return true;
	}
	
	public void clearSentence() {
		size = 0;
	}
	
	public String toString() {
		StringBuilder sb = new StringBuilder();
		
		for (int i=1;i<=size;i++) {
			sb.append(tokens[i]+"\n");
		}
		return sb.toString();
	}
}
