package org.maltparser.core.symbol.trie;

import org.maltparser.core.symbol.SymbolException;

/**

@author Johan Hall
@since 1.0
*/
public class Trie {
	private TrieNode root;
	private StringBuilder sb;
	
	public Trie() {
		root = new TrieNode(' ', null);
		sb = new StringBuilder();
	}
	
	public TrieNode addValue(String value, TrieSymbolTable table, int code) throws SymbolException {
		TrieNode node = root;
		for (int i=value.length()-1;i>=0;i--) {
			if (i == 0) {
				node = node.getOrAddChild(true, value.charAt(i), table, code);
			} else {
				node = node.getOrAddChild(false, value.charAt(i), table, code);
			}
		}
		return node;
	}
	
	public String getValue(TrieNode node, TrieSymbolTable table) {
		sb.setLength(0);
		TrieNode tmp = node;
		while (tmp != root && tmp != null) {
			sb.append(tmp.getCharacter());
			tmp = tmp.getParent();
		}
		return sb.toString();
	}
	
	public TrieEntry getEntry(String value, TrieSymbolTable table) {
		TrieNode node = root;
		int i=value.length()-1;
		for (;i>=0 && node != null;i--) {
			node = node.getChild(value.charAt(i));
		}
		if (i < 0 && node != null) {
			return node.getEntry(table);
		} 
		return null;
	}
	
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (!(obj instanceof Trie)) {
			return false;
		}
		Trie objT = (Trie)obj;
		
		return ((root == null) ? objT.root == null : root.equals(objT.root));
	}

	public int hashCode() {
		int hash = 7;
		hash = 31 * hash + (null == root ? 0 : root.hashCode());
		return hash;
	}
}
