package org.maltparser.parser.algorithm.nivre;


import org.maltparser.core.config.SingleMaltConfiguration;
import org.maltparser.core.exception.MaltChainedException;
import org.maltparser.core.graph.DependencyGraph;
import org.maltparser.core.sentence.Sentence;

import org.maltparser.parser.algorithm.helper.ParserAction;
import org.maltparser.parser.algorithm.helper.Transition;

/**
 * 
 * @author Joakim Nivre
 * @author Johan Hall
 * @since 1.0
*/
public class NivreStandard extends Nivre {
	protected static final int RIGHTARC = 2;
	protected static final int LEFTARC = 3;
	
	public NivreStandard(SingleMaltConfiguration configuration, Sentence sentence) throws MaltChainedException {
		super(configuration, sentence);
	}
	
	protected void transition(DependencyGraph dg) throws MaltChainedException {

		while (checkParserAction(dg) == false) {
			if (configuration.getMode() == SingleMaltConfiguration.LEARN || configuration.predictFromKBestList() == false) {
				parserAction.replaceAction(SHIFT); // default parser action
				break;
			}
		}

		switch (parserAction.getLastTransition().getCode()) {
		case LEFTARC:
			dg.addArc(input.peek(), stack.peek(), parserAction.getLastLabelCode());
			stack.pop();
			break;
		case RIGHTARC:
			dg.addArc(stack.peek(), input.peek(), parserAction.getLastLabelCode());
			input.pop();
			if (!stack.peek().isRoot()) {
				input.push(stack.pop());	
			}
			break;
		default:
			stack.push(input.pop()); // SHIFT
			break;
		}
	}
	
	protected boolean checkParserAction(DependencyGraph dg) throws MaltChainedException {
		int trans = parserAction.getLastTransition().getCode();
	
		if ((trans == LEFTARC || trans == RIGHTARC) && parserAction.getLastLabelCode() == null) {
			// Label code is null for transitions LEFTARC and RIGHTARC
			return false;
		}
		/* if (trans == SHIFT && parserAction.getLastLabelCode() != null) {
			// Label code is not null for transition SHIFT
		}*/
		if (trans == LEFTARC && stack.peek().isRoot()) { 
			// The token on top of the stack is root for transition LEFTARC
			return false;
		}
		return true;
	}

	
	protected void oraclePredict(DependencyGraph gold) throws MaltChainedException {
		if (!stack.peek().isRoot() && gold.getTokenNode(stack.peek().getIndex()).getHead().getIndex().intValue() == input.peek().getIndex().intValue()) {
			parserAction.addAction(LEFTARC, gold.getTokenNode(stack.peek().getIndex()).getLabelCode());
		} else if (gold.getTokenNode(input.peek().getIndex()).getHead().getIndex().intValue() == stack.peek().getIndex().intValue() && checkRightDependent(gold)) {
			parserAction.addAction(RIGHTARC, gold.getTokenNode(input.peek().getIndex()).getLabelCode());
		} else {
			parserAction.addAction(SHIFT);
		}
	}

	private boolean checkRightDependent(DependencyGraph gold) {
		if (gold.getTokenNode(input.peek().getIndex()).getRightmostDependent() == null) {
			return true;
		} else if (dependencyGraph.getTokenNode(input.peek().getIndex()).getRightmostDependent() != null) {
			if (gold.getTokenNode(input.peek().getIndex()).getRightmostDependent().getIndex().intValue() == dependencyGraph.getTokenNode(input.peek().getIndex()).getRightmostDependent().getIndex().intValue()) {
				return true;
			}
		}
		return false;
	}
	
	protected void initParserAction() {
		Transition[] transitions = new Transition[3];

		transitions[0] = new Transition(SHIFT,"SH",false);
		transitions[1] = new Transition(RIGHTARC,"RA",true);
		transitions[2] = new Transition(LEFTARC,"LA",true);

		parserAction = new ParserAction(transitions, this);	
	}
	
	public String getName() {
		return "nivrestandard";
	}
}