package org.maltparser.parser.guide.feature;

import org.maltparser.core.dataformat.ColumnDescription;
import org.maltparser.core.exception.MaltChainedException;
import org.maltparser.core.graph.Node;
import org.maltparser.core.symbol.SymbolTable;
import org.maltparser.core.symbol.nullvalue.NullValues.NullValueId;

/**


@author Johan Hall
@since 1.0
 */
public abstract class ColumnFeature extends TokenFeature {
	protected ColumnDescription column;
	private String specification;
	protected Node currentUpdateNode;
	
	public ColumnFeature() throws MaltChainedException {
		super();
	}
	
	public boolean init(String spec) throws MaltChainedException {
		specification = spec.trim();
		super.setCardinality(column.getSymbolTable().getValueCounter()-1);
		return createOperationStack(specification);
	}
	
	public abstract void update() throws MaltChainedException;
	
	public String getStringValue(Integer value) throws MaltChainedException {
		return column.getSymbolTable().getSymbolCodeToString(value);
	}
	
	public Integer getIntegerValue(String value) throws MaltChainedException {
		return column.getSymbolTable().getSymbolStringToCode(value);
	}
	
	public ColumnDescription getColumn() {
		return column;
	}
	
	protected void setColumn(ColumnDescription column) {
		this.column = column;
	}
	
	public void updateCardinality() {
		super.setCardinality(column.getSymbolTable().getValueCounter()); 
	}
	
	public void setValue(Integer value) throws MaltChainedException {
		featureValues.clear();
		featureKnown.clear();
		if (column.getSymbolTable().getSymbolCodeToString(value) == null) {
			featureKnown.put(value, column.getSymbolTable().getKnown(value));
			featureValues.put(value, column.getSymbolTable().getNullValueSymbol(NullValueId.NO_NODE));
			nullValue = true;
		} else {
			featureKnown.put(value, column.getSymbolTable().getKnown(value));
			featureValues.put(value, column.getSymbolTable().getSymbolCodeToString(value));
			nullValue = column.getSymbolTable().isNullValue(value);
		}
	}
	
	public void setValue(String value) throws MaltChainedException {
		featureValues.clear();
		featureKnown.clear();
		if (column.getSymbolTable().getSymbolStringToCode(value) < 0) {
			featureKnown.put(column.getSymbolTable().getNullValueCode(NullValueId.NO_NODE), column.getSymbolTable().getKnown(value));
			featureValues.put(column.getSymbolTable().getNullValueCode(NullValueId.NO_NODE), value);
			nullValue = true;
		} else {
			featureKnown.put(column.getSymbolTable().getSymbolStringToCode(value), column.getSymbolTable().getKnown(value));
			featureValues.put(column.getSymbolTable().getSymbolStringToCode(value), value);
			nullValue = column.getSymbolTable().isNullValue(value);
		}
	}
	
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (!(obj instanceof ColumnFeature)) {
			return false;
		}
		return obj.toString().equals(toString());
	}

	public int hashCode() {
		int hash = 7;
		hash = 31 * hash + (null == toString() ? 0 : toString().hashCode());
		return hash;
	}
	
	public String getColumnName() {
		return column.getName();
	}
	
	public SymbolTable getSymbolTable() {
		return column.getSymbolTable();
	}
	
	public Node getCurrentUpdateNode() {
		return currentUpdateNode;
	}
	
	public String toString() {
		final StringBuilder sb = new StringBuilder();
		sb.append(column.getName());
		sb.append(", ");
		sb.append(specification);
		return sb.toString();
	}
}
