package org.maltparser.parser.guide.feature;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;

import org.maltparser.core.exception.MaltChainedException;
import org.maltparser.parser.algorithm.ParsingAlgorithm;

/**


@author Johan Hall
@since 1.0
 */
public abstract class Feature {
	protected SortedMap<Integer, String> featureValues;
	protected int cardinality;
	protected Map<Integer, Boolean> featureKnown;
	protected boolean nullValue;
	protected ParsingAlgorithm parsingAlgorithm;
	
	public Feature() throws MaltChainedException {
		cardinality = 0;
		nullValue = false;
		parsingAlgorithm = null;
	}

	public void init() throws MaltChainedException {
		this.initialize(null);
	}
	
	public void initialize(ParsingAlgorithm parsingAlgorithm) throws MaltChainedException {
		setParsingAlgorithm(parsingAlgorithm);
		/*if (parsingAlgorithm != null && (Boolean)parsingAlgorithm.getConfiguration().getOptionValue("malt0.4", "behavior") == true) {
			featureValues = new LinkedHashMap<Integer, String>();
		} else {*/
			featureValues = new TreeMap<Integer, String>();
		//}
		featureKnown = new HashMap<Integer, Boolean>();
	}
	
	protected void setParsingAlgorithm(ParsingAlgorithm parsingAlgorithm) {
		this.parsingAlgorithm = parsingAlgorithm;
	}
	/**
	 * Returns the cardinality (the number of distinct values) of the feature 
	 * 
	 * @return the cardinality (the number of distinct values) of the feature 
	 */
	public int getCardinality() {
		return cardinality;
	}

	/**
	 * Sets the cardinality (the number of distinct values) of the feature
	 * 
	 * @param cardinality  the cardinality (the number of distinct values)
	 */
	public void setCardinality(int cardinality) {
		this.cardinality = cardinality;
	}

	public abstract void update() throws MaltChainedException;
	public abstract boolean init(String spec) throws MaltChainedException;
	public abstract void updateCardinality() throws MaltChainedException;
	public abstract String getStringValue(Integer value) throws MaltChainedException;
	public abstract Integer getIntegerValue(String value) throws MaltChainedException;	
	public abstract void setValue(Integer value) throws MaltChainedException;
	public abstract void setValue(String value) throws MaltChainedException;
	
	public Map<Integer, String> getValues() {
		return featureValues;
	}
	
	public Set<Integer> getIntegerValues() {
		return (Set<Integer>)featureValues.keySet();
	}
	
	public Integer getFirstIntegerValue() {
		return featureValues.firstKey();
		/*if (featureValues instanceof TreeMap) {
			return ((TreeMap<Integer, String>)featureValues).firstKey();
		} else if (featureValues instanceof LinkedHashMap) {
			for (Integer index : ((LinkedHashMap<Integer, String>)featureValues).keySet()) {
				return index;
			}
		}
		return null;*/
	}
	
	public Set<String> getStringValues() {
		return new HashSet<String>(featureValues.values());
	}
	
	public String getFirstStringValue() {
		return featureValues.get(featureValues.firstKey());
		/*if (featureValues instanceof TreeMap) {
			return featureValues.get(((TreeMap<Integer, String>)featureValues).firstKey());
		} else if (featureValues instanceof LinkedHashMap) {
			for (Integer index : ((LinkedHashMap<Integer, String>)featureValues).keySet()) {
				return featureValues.get(index);
			}
		}
		return null;*/
	}	
	
	/* public boolean isFirstKnown() {
		if (featureKnown instanceof TreeMap) {
			return featureKnown.get(((TreeMap<Integer, Boolean>)featureKnown).firstKey());
		} else if (featureKnown instanceof LinkedHashMap) {
			for (Integer index : ((LinkedHashMap<Integer, Boolean>)featureKnown).keySet()) {
				return featureKnown.get(index);
			}
		}
		return false;
	}*/
	
	public boolean isKnown(int value) {
		return featureKnown.get(value);
	}
	
	public boolean isNullValue() {
		return nullValue;
	}
	
	protected ParsingAlgorithm getParsingAlgorithm() {
		return parsingAlgorithm;
	}
	
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (!(obj instanceof Feature)) {
			return false;
		}
		return true;
	}
	
	public String toString() {
		return "";
	}

}
