package org.maltparser.parser.guide.feature;

import org.maltparser.core.exception.MaltChainedException;
import org.maltparser.parser.algorithm.ParsingAlgorithm;
import org.maltparser.parser.algorithm.helper.ParserAction;


/**
TODO: NB not tested yet, probably wrong behavior

@author Johan Hall
@since 1.0
 */
public class ParserActionFeature extends AtomicFeature {
	private ParserAction parserAction;
	private int index;
	private String specification;
	private String arcName;
	
	public ParserActionFeature(ParsingAlgorithm parsingAlgorithm, String spec) throws MaltChainedException {
		super();
		super.initialize(parsingAlgorithm);
		parserAction = parsingAlgorithm.getParserAction();
		spec = spec.trim();
		int index = spec.indexOf(',');
		if (index == -1) {
			throw new FeatureException("The parser action feature '"+spec+"' has not the correct format. ");
		}
		arcName = spec.substring(0, index).trim();
		if (parsingAlgorithm.getParserAction().getOutputSymbolTables().containsKey(arcName) || arcName.equals("TRANS")) {
			if (!init(spec.substring(index+1).trim())) { 
				throw new FeatureException("The parser action feature could not be initialized at line '"+spec+"'. ");
			}
		} else {
			throw new FeatureException("The output symbol table '"+arcName+"' could not be found, when initializing the feature '"+spec+"'. ");	
		}
		updateCardinality();
	}

	public boolean init(String spec) {
		specification = spec;
		try {
			index = Integer.parseInt(spec);
			if (index < 0) {
				return false;
			}
		} catch (NumberFormatException e) {
			return false;
		}
		
		return true;
	}
	
	public void update() throws MaltChainedException {
		featureValues.clear();
		featureValues.put(parserAction.getLabelCode(parserAction.getOutputSymbolTable(arcName), index), parserAction.getLabel(parserAction.getOutputSymbolTable(arcName), index));
		nullValue = false;
	}
	
	public void setValue(Integer value) throws MaltChainedException {
		featureValues.clear();
		featureValues.put(value, parserAction.getOutputSymbolTable(arcName).getSymbolCodeToString(value));
		nullValue = false;
	}
	
	public void setValue(String value) throws MaltChainedException {
		featureValues.clear();
		featureValues.put(parserAction.getOutputSymbolTable(arcName).getSymbolStringToCode(value), value);
		nullValue = false;
	}
	
	
	public String getStringValue(Integer value) throws MaltChainedException {
		return parserAction.getOutputSymbolTable(arcName).getSymbolCodeToString(value);
	}
	
	public Integer getIntegerValue(String value) throws MaltChainedException {
		return parserAction.getOutputSymbolTable(arcName).getSymbolStringToCode(value);
	}
	
	public void updateCardinality() {
		super.setCardinality(parserAction.getCardinality(arcName));
	}

	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (!(obj instanceof ParserActionFeature)) {
			return false;
		}
		return obj.toString().equals(toString());
	}
	
	public int hashCode() {
		int hash = 7;
		hash = 31 * hash + (null == toString() ? 0 : toString().hashCode());
		return hash;
	}
	
	public String toString() {
		final StringBuilder sb = new StringBuilder();
		sb.append("ParserAction(");
		sb.append(arcName);
		sb.append(", ");
		sb.append(specification);
		sb.append(')');
		return sb.toString();
	}
}
